#!/usr/bin/env python3

import os
import sys
import subprocess as sp


"""
unmute.py: replicate stdout/stderr to a parent process
2020-03-21, ed <irc.rizon.net>, MIT-Licensed
https://ocv.me/dev/?gist/unmute.py

affects all(?) python code but not subprocess'd stuff

can be somewhat replicated in bash scripts by
doing a redirect rather than tee:
  exec <$(tty) >$(tty) 2>$(tty)
"""


class Tee(object):
    def __init__(self, sinks, top=True):
        self.sinks = [x for x in sinks if x]
        if top:
            self.buffer = Tee(self.sinks, False)
            self.encoding = self.sinks[0].encoding
        else:
            self.sinks = [x.buffer for x in self.sinks]

    def write(self, *args, **kwargs):
        for sink in self.sinks:
            sink.write(*args, **kwargs)

    def flush(self, *args, **kwargs):
        for sink in self.sinks:
            sink.flush(*args, **kwargs)


def unmute():
	# expect these somewhere in the cmdline of 1st and 2nd parent process,
	# then fork stdout/stderr to FD 1/2 of the last parent
    parents = [
        '/var/tmp/rpm-tmp',
        '/bin/yum '
    ]
    ppid = os.getppid()
    for substr in parents:
        cmd = 'ps -o ppid= -o pid= -o command= -p ' + str(ppid)
        txt = sp.check_output(cmd.split(' '))
        txt = txt.decode('utf-8', 'ignore').strip()
        while '  ' in txt:
            txt = txt.replace('  ', ' ')

        ppid, pid, cmd = txt.split(' ', 2)
        if substr not in cmd:
            pid = None
            break

    if pid is None:
        return

    if sys.stdout and sys.stdout.buffer:
        f = open('/proc/{}/fd/1'.format(pid), 'w', encoding=sys.stdout.encoding)
        sys.stdout = Tee([sys.stdout, f])

    if sys.stderr and sys.stderr.buffer:
        f = open('/proc/{}/fd/2'.format(pid), 'w', encoding=sys.stderr.encoding)
        sys.stderr = Tee([sys.stderr, f])


if __name__ == '__main__':
    import time

    print('stdout before')
    print('stderr before', file=sys.stderr)
    unmute()
    print('stdout after')
    print('stderr after', file=sys.stderr)
    time.sleep(2)

